"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.baseQueryWithRefresh = exports.baseQuery = void 0;
const async_mutex_1 = require("async-mutex");
const slice_1 = require("@procyon/auth/slice");
const types_1 = require("@procyon/auth/types");
const utils_1 = require("@procyon/utils");
const utils_2 = require("./utils");
const mutex = new async_mutex_1.Mutex();
const baseQuery = ({ baseUrl, authentication, }) => ({ url, method = 'GET', params, body, headers }, { getState }) => __awaiter(void 0, void 0, void 0, function* () {
    if (authentication && getState().procyon[types_1.AUTH_REDUCER_NAME] !== undefined) {
        const isAuthenticated = (0, slice_1.getAuthAttribute)(getState(), 'authenticated');
        if (isAuthenticated) {
            const tokenValue = (0, slice_1.getAuthAttribute)(getState(), 'accessToken');
            headers = Object.assign(Object.assign({}, headers), { [authentication.headerName]: `${authentication.tokenType} ${tokenValue === null || tokenValue === void 0 ? void 0 : tokenValue.toString()}`.trim() });
        }
    }
    try {
        const response = yield fetch(`${baseUrl}${(0, utils_2.buildUrl)(url, params)}`, Object.assign({ headers: Object.assign({ 'Content-Type': 'application/json' }, headers), method: method.toUpperCase() }, (body ? { body: new URLSearchParams(body) } : {})));
        const json = yield response.text().then((text) => ((0, utils_1.isJSON)(text) ? JSON.parse(text) : text));
        if (!response.ok)
            throw response;
        return { data: json };
    }
    catch (err) {
        return { error: { status: err.status, error: err } };
    }
});
exports.baseQuery = baseQuery;
const baseQueryWithRefresh = ({ baseUrl, authentication, }) => (args, api, extraOptions) => __awaiter(void 0, void 0, void 0, function* () {
    const authReducer = api.getState().procyon[types_1.AUTH_REDUCER_NAME];
    const accessTokenExpired = new Date(authReducer.accessExpiresTime) < new Date();
    if (accessTokenExpired) {
        yield mutex.waitForUnlock();
        const refreshTokenExpired = new Date(authReducer.refreshExpiresTime) < new Date();
        if (!refreshTokenExpired) {
            if (!mutex.isLocked()) {
                const release = yield mutex.acquire();
                try {
                    const { data: tokens } = yield (0, exports.baseQuery)({ baseUrl: authentication.baseUrl, authentication })({
                        url: authentication.tokenEndpoint,
                        method: 'post',
                        headers: { 'Content-Type': 'application/x-www-form-urlencoded' },
                        body: {
                            client_id: authentication.clientId,
                            grant_type: 'refresh_token',
                            refresh_token: authReducer.refreshToken,
                        },
                    }, api, extraOptions);
                    api.dispatch((0, slice_1.setAuthTokens)({
                        accessToken: tokens.access_token,
                        refreshToken: tokens.refresh_token,
                        idToken: tokens.id_token,
                        accessExpiresIn: tokens.expires_in,
                        refreshExpiresIn: tokens.refresh_expires_in,
                        sessionState: tokens.session_state,
                    }));
                }
                finally {
                    release();
                }
            }
        }
        else {
            api.dispatch((0, slice_1.setAuthAttribute)({ authenticated: false }));
            window.location.assign(authentication.redirectLoginUri);
            return null;
        }
    }
    return yield (0, exports.baseQuery)({ baseUrl, authentication })(args, api, extraOptions);
});
exports.baseQueryWithRefresh = baseQueryWithRefresh;
